import os
import sys
import shutil
import subprocess
import time
import numpy as np

#==============================================================================
# CHEMKINDOS.py - given chemin chemistry file containing both chemistry and thermo data
# as well as a reactin input file for CHEMKIN closed homogeneous batch reactor (0-D reactor)
# Set up CHEMKIN environment variables (specified by envvars files), pre-process the input,
# run the reactor, and post-process
#==============================================================================

def runshock(chemin="chem.inp",thermin="therm.dat",reactin="C:\\Python_Codes\\Chemkintest\\homotest1.inp",envvars="chemkinpro_env.txt",work_dir=os.getcwd(),getsolnlist=False):
    os.chdir(work_dir)
    print('Running task in: '+os.getcwd())
    #check if chemkin exists
    if os.path.exists(chemin)==False:
        print(chemin+" not found in working directory, check file name")
        sys.exit()
    #check if reactin exists
    if os.path.exists(thermin)==False:
        print(thermin+" not found in working directory, check file name")
        sys.exit()

    if os.path.exists(reactin)==False:
        print(reactin+" not found in working directory, check file name")
        sys.exit()
    #check if envvars exists. This batch script sets up all necessary CHEMKIN/ enviornment
    #variables necessary to perform the following commands
    if os.path.exists(envvars)==False:
        sys.exit(envvars+" not found in working directory, check file name")
    
    #Check if CKSolnList.txt exists in wd. Necessary for post-processing options
    if os.path.exists(envvars)==False:
        sys.exit("CKSolnList.txt not found in working directory, check file name")        

    #set environment variables needed by CHEMKIN
    #These come from the the envvars file where each line is VAR=value
    try:
        with open(envvars, "r") as envFile:
            for linevar in envFile:
                #Read the var name and value into VAR, VAL by
                #first stripping line of any eol values and then splitting across
                #"=" just once
                VAR,VAL = linevar.rstrip().split("=",1)

                #set environment VAR to VAL
                os.environ[VAR]=VAL
                print(os.environ[VAR])

                #if VAR is CHEMKIN_BIN, prepend to PATH variable
#                if VAR=="CHEMKIN_BIN":
#                    os.environ['PATH'] += os.pathsep + VAL
#                    print(os.getenv('PATH')+"\n")

                print("Setting: "+VAR+"="+VAL)
                print("Cheking: "+VAR+" is set to: "+os.getenv(VAR))
    except:
        print("\n ERROR setting up environmental variables, check env var. file is \
formatted correctly. Make sure there are no blank lines. Format is VAR=value. \n")
        raise

    #chemkindata.dtd copy from %CHEMKIN_DATA%
    filepathdt=r"C:\Python_Codes\Chemkintest\chemkindata.dtd"
    if os.path.exists("chemkindata.dtd")==0:
        try:
            print("Copying chemkindata.dtd template from %CHEMKIN_DATA% to working dir...")
            shutil.copy(filepathdt,os.getcwd())
        except IOError:
            print("\nERROR: chemkindata.dtd file not found in path given: "+filepathdt)
            print("\nEnsure that the file exists in CHEMKIN_DATA directory \n")
            #raise
        except:
            print("Unexpected error copying chemkin.dtd")
            raise
    
    
    #execute system command to pre-process
    
#==============================================================================
#     this part is a bit of a mess. subprocess is preferred over using os.system
#     additionally, copying files above and running chemkin using environmental variables requires
#     the following command to be executed in "Shell" mode
#     However, looking at the documents for subprocess, running things in "Shell" mode is a bit of a
#     security risk and is not recommended. Still need a find a "proper" way of doing this
#     Couldn't figure out the input output issues with subprocess
#     but this works for now
#==============================================================================
    print(os.environ['PATH'])
    pre_try = 1
    while pre_try<=5: #this is needed for network failures
        #here
        try:
            print("Pre-processing the input file... Try: "+str(pre_try)+" of 5")
            pre_try+=1
#            subprocess.check_call('"%CHEMKIN_BIN%"\\chem -i '+chemin+' -o chem.out \\ -d '+thermin,shell=True)
#            ccc='"%CHEMKIN_BIN%"\\chem -i ',chemin,' -o chem.out \\ -d ',thermin
            subprocess.check_call(r'C:\Program Files\ANSYS Inc\v180\reaction\chemkinpro.win64\bin\chem -i '+chemin+r' -o chem.out \ -d '+thermin, shell=True)
            
            print("Done pre-processing.")
            break
        except:
            print("Unexpected error running CHEMKIN pre-processor.")
            print("Trying again in 10 seconds")
            print(os.environ['CHEMKIN_BIN'])
            time.sleep(10)
            raise
    
    if pre_try > 5: #end exuction 
        print("Maximum number of tries for pre-processing reached. Exiting.")
        sys.exit()

        
#==============================================================================
# Run CHEMKIN CKReactorGenericClosed located in the %CHEMKIN_BIN%
# The file closed_homog.inp needs to be in the working directory
# and formmated correctly. A "template" can be created using the CHEMKIN gui.
# Also working chemistry link-file should have been created by the pre-processor
# If there were any issues in pre-processing then this link-file would not have 
# been created (chem.asc). Presently, this script doesn't check whether 
# pre-processor was successful. If there were any problems, 
# CHEMKIN writes them to the chem.out file. In future it may be worth while to 
# add an efficient of checking for errors. chem.out files are long and it's a
# waste of time to search line by line
#==============================================================================
    react_try=1
    while react_try<=5: #this is needed for network failures
        try:
            react_try+=1
            if os.path.exists('chem.asc')==False:
                print("chem.asc not found in working directory, check file name")
                sys.exit()
            print("Running closed_homogenous reactor... Try: "+str(react_try-1)+" of 5") 
            subprocess.call(r'C:\Program Files\ANSYS Inc\v180\reaction\chemkinpro.win64\bin\CKreactorGenericClosed -i '+reactin+r" -o closed_homog.out",shell=True)
            print("Finished running.")
            break
        except:
            print("Unexpected error running CKReactorGenericClosed...")
            print("Trying again in 10 seconds.")
            time.sleep(10)
            continue
        
    if react_try > 5: #end exuction 
        print("Maximum number of tries for running the reactor reached. Exiting.")
        sys.exit()

#Post process the results  
    pp_try=1
    while pp_try<=5:    #this is needed for network failures
        try:
            print("Running CHEMKIN post-processor... Try: "+str(pp_try)+" of 5")
            pp_try+=1
            
            #If getsolnlist is true, runs the model ones and generates a CKSolnList.txt 
            #Necessary to only pull out data for specific species
            if getsolnlist:
                subprocess.call(r'C:\Program Files\ANSYS Inc\v180\reaction\chemkinpro.win64\bin\GetSolution -listonly XMLdata.zip',shell=True)
                print('Created a CKSolnList.txt, toggle getsolnlist=False for subsequent runs')
                break
            
            #post process regularly, if a CKSolnList.txt is present, chemkin will only
            #pull out the specificed data
            subprocess.call(r'C:\Program Files\ANSYS Inc\v180\reaction\chemkinpro.win64\bin\GetSolution XMLdata.zip',shell=True)
            print("Finished running.")
            break
        except:
            print("Unexpected error running CHEMKIN post-processor...")
            print("Trying again in 10 seconds.")
            time.sleep(10)
            continue

    if pp_try > 5: #end exuction 
        print("Maximum number of tries for post-processing reached. Exiting.")
        sys.exit()



def runshockafterpreproc(reactin="C:\\Python_Codes\\Chemkintest\\homotest1.inp",envvars="chemkinpro_env.txt",work_dir=os.getcwd(),getsolnlist=False):
    os.chdir(work_dir)
    print('Running task in: '+os.getcwd())
    #check if chemkin exists
    if os.path.exists("chem.asc")==False:
        print("chem.asc not found in working directory, check file name")
        sys.exit()
    #check if reactin exists
#    if os.path.exists(thermin)==False:
#        print(thermin+" not found in working directory, check file name")
#        sys.exit()

    if os.path.exists(reactin)==False:
        print(reactin+" not found in working directory, check file name")
        sys.exit()
    #check if envvars exists. This batch script sets up all necessary CHEMKIN/ enviornment
    #variables necessary to perform the following commands
    if os.path.exists(envvars)==False:
        sys.exit(envvars+" not found in working directory, check file name")
    
    #Check if CKSolnList.txt exists in wd. Necessary for post-processing options
    if os.path.exists(envvars)==False:
        sys.exit("CKSolnList.txt not found in working directory, check file name")        

    #set environment variables needed by CHEMKIN
    #These come from the the envvars file where each line is VAR=value
    try:
        with open(envvars, "r") as envFile:
            for linevar in envFile:
                #Read the var name and value into VAR, VAL by
                #first stripping line of any eol values and then splitting across
                #"=" just once
                VAR,VAL = linevar.rstrip().split("=",1)

                #set environment VAR to VAL
                os.environ[VAR]=VAL

                #if VAR is CHEMKIN_BIN, prepend to PATH variable
                if VAR=="CHEMKIN_BIN":
                    os.environ['PATH'] += os.pathsep + VAL
                    #print(os.getenv('PATH')+"\n")

                #print("Setting: "+VAR+"="+VAL)
                #print("Cheking: "+VAR+" is set to: "+os.getenv(VAR))
    except:
        print("\n ERROR setting up environmental variables, check env var. file is \
formatted correctly. Make sure there are no blank lines. Format is VAR=value. \n")
        raise

    #chemkindata.dtd copy from %CHEMKIN_DATA%
    filepathdt=r"C:\Python_Codes\Chemkintest\chemkindata.dtd"
    if os.path.exists("chemkindata.dtd")==0:
        try:
            print("Copying chemkindata.dtd template from %CHEMKIN_DATA% to working dir...")
            shutil.copy(filepathdt,os.getcwd())
        except IOError:
            print("\nERROR: chemkindata.dtd file not found in path given: "+filepathdt)
            print("\nEnsure that the file exists in CHEMKIN_DATA directory \n")
            #raise
        except:
            print("Unexpected error copying chemkin.dtd")
            raise
    
    
    #execute system command to pre-process

        
#==============================================================================
# Run CHEMKIN CKReactorGenericClosed located in the %CHEMKIN_BIN%
# The file closed_homog.inp needs to be in the working directory
# and formmated correctly. A "template" can be created using the CHEMKIN gui.
# Also working chemistry link-file should have been created by the pre-processor
# If there were any issues in pre-processing then this link-file would not have 
# been created (chem.asc). Presently, this script doesn't check whether 
# pre-processor was successful. If there were any problems, 
# CHEMKIN writes them to the chem.out file. In future it may be worth while to 
# add an efficient of checking for errors. chem.out files are long and it's a
# waste of time to search line by line
#==============================================================================
    react_try=1
    while react_try<=5: #this is needed for network failures
        try:
            react_try+=1
            if os.path.exists('chem.asc')==False:
                print("chem.asc not found in working directory, check file name")
                sys.exit()
            print("Running closed_homogenous reactor... Try: "+str(react_try-1)+" of 5") 
            subprocess.call('"%CHEMKIN_BIN%"\CKreactorGenericClosed -i '+reactin+" -o closed_homog.out",shell=True)
            print("Finished running.")
            break
        except:
            print("Unexpected error running CKReactorGenericClosed...")
            print("Trying again in 10 seconds.")
            time.sleep(10)
            continue
        
    if react_try > 5: #end exuction 
        print("Maximum number of tries for running the reactor reached. Exiting.")
        sys.exit()

#Post process the results  
    pp_try=1
    while pp_try<=5:    #this is needed for network failures
        try:
            print("Running CHEMKIN post-processor... Try: "+str(pp_try)+" of 5")
            pp_try+=1
            
            #If getsolnlist is true, runs the model ones and generates a CKSolnList.txt 
            #Necessary to only pull out data for specific species
            if getsolnlist:
                subprocess.call('"%CHEMKIN_BIN%"\GetSolution -listonly XMLdata.zip',shell=True)
                print('Created a CKSolnList.txt, toggle getsolnlist=False for subsequent runs')
                break
            
            #post process regularly, if a CKSolnList.txt is present, chemkin will only
            #pull out the specificed data
            subprocess.call('"%CHEMKIN_BIN%"\GetSolution XMLdata.zip',shell=True)
            print("Finished running.")
            break
        except:
            print("Unexpected error running CHEMKIN post-processor...")
            print("Trying again in 10 seconds.")
            time.sleep(10)
            continue

    if pp_try > 5: #end exuction 
        print("Maximum number of tries for post-processing reached. Exiting.")
        sys.exit()
        

def postprocess(envvars="chemkinpro_env.txt",work_dir=os.getcwd(),getsolnlist=False):
    os.chdir(work_dir)
    print('Running task in: '+os.getcwd())
    #check if chemkin exists
    if os.path.exists("chem.asc")==False:
        print("chem.asc not found in working directory, check file name")
        sys.exit()
    #check if reactin exists
#    if os.path.exists(thermin)==False:
#        print(thermin+" not found in working directory, check file name")
#        sys.exit()

    #check if envvars exists. This batch script sets up all necessary CHEMKIN/ enviornment
    #variables necessary to perform the following commands
    if os.path.exists(envvars)==False:
        sys.exit(envvars+" not found in working directory, check file name")
    
    #Check if CKSolnList.txt exists in wd. Necessary for post-processing options
    if os.path.exists("CKSolnList.txt")==False:
        sys.exit("CKSolnList.txt not found in working directory, check file name")        

    #set environment variables needed by CHEMKIN
    #These come from the the envvars file where each line is VAR=value
    try:
        with open(envvars, "r") as envFile:
            for linevar in envFile:
                #Read the var name and value into VAR, VAL by
                #first stripping line of any eol values and then splitting across
                #"=" just once
                VAR,VAL = linevar.rstrip().split("=",1)

                #set environment VAR to VAL
                os.environ[VAR]=VAL
                print(os.environ[VAR])

                #if VAR is CHEMKIN_BIN, prepend to PATH variable
#                if VAR=="CHEMKIN_BIN":
#                    os.environ['PATH'] += os.pathsep + VAL
#                    print(os.getenv('PATH')+"\n")

                print("Setting: "+VAR+"="+VAL)
                print("Cheking: "+VAR+" is set to: "+os.getenv(VAR))
    except:
        print("\n ERROR setting up environmental variables, check env var. file is \
formatted correctly. Make sure there are no blank lines. Format is VAR=value. \n")
        raise



#Post process the results  
    pp_try=1
    while pp_try<=5:    #this is needed for network failures
        try:
            print("Running CHEMKIN post-processor... Try: "+str(pp_try)+" of 5")
            pp_try+=1
            
            #If getsolnlist is true, runs the model ones and generates a CKSolnList.txt 
            #Necessary to only pull out data for specific species
            if getsolnlist:
                subprocess.call(r'C:\Program Files\ANSYS Inc\v180\reaction\chemkinpro.win64\bin\GetSolution -listonly XMLdata.zip',shell=False)
                print('Created a CKSolnList.txt, toggle getsolnlist=False for subsequent runs')
                break
            
            #post process regularly, if a CKSolnList.txt is present, chemkin will only
            #pull out the specificed data
            subprocess.call(r'C:\Program Files\ANSYS Inc\v180\reaction\chemkinpro.win64\bin\GetSolution XMLdata.zip',shell=False)
            print("Finished running.")
            break
        except:
            print("Unexpected error running CHEMKIN post-processor...")
            print("Trying again in 10 seconds.")
            time.sleep(10)
            continue

    if pp_try > 5: #end exuction 
        print("Maximum number of tries for post-processing reached. Exiting.")
        sys.exit()




def singlespeciesreaction(A,ER,t,T,fraction):
    Ex=np.exp(-ER/T)
    J=np.trapz(Ex,x=t)
    X=1-1/np.exp(A*J)
    return X


    
    

"""allows script to be imported without execution"""
if __name__ == '__main__':
    runshockafterpreproc(reactin=r'C:\Users\Xu\Desktop\Changing pressure simulation\Simulation_results\Shock104\\Shock104.inp',
                 envvars="chemkinpro_env.txt",work_dir=r'C:\Users\Xu\Desktop\Changing pressure simulation\Simulation_results\Shock104',
                 getsolnlist=False)
